// __multiversion__
// This signals the loading code to prepend either #version 100 or #version 300 es as apropriate.

#include "vertexVersionCentroid.h"
#if __VERSION__ >= 300
	#ifndef BYPASS_PIXEL_SHADER
		_centroid out vec2 uv0;
		_centroid out vec2 uv1;
	#endif
#else
	#ifndef BYPASS_PIXEL_SHADER
		varying vec2 uv0;
		varying vec2 uv1;
	#endif
#endif

#ifndef BYPASS_PIXEL_SHADER
	varying vec4 color;
	varying highp vec3 position, wpos;
	varying highp vec2 screenPos;
#endif

#ifdef FOG
	varying vec4 fogColor;
#endif

#include "uniformWorldConstants.h"
#include "uniformPerFrameConstants.h"
#include "uniformShaderConstants.h"
#include "uniformRenderChunkConstants.h"

attribute POS4 POSITION;
attribute vec4 COLOR;
attribute vec2 TEXCOORD_0;
attribute vec2 TEXCOORD_1;

const float rA = 1.0;
const float rB = 1.0;
const vec3 UNIT_Y = vec3(0,1,0);
const float DIST_DESATURATION = 56.0 / 255.0; //WARNING this value is also hardcoded in the water color, don'tchange

#include "include/condition.h"

float customLeafWave(float speed)
{
	return sin(TIME * speed + position.x * 2.0 - position.y + position.z + cos(TIME * 1.9 + position.x * 0.6 - position.y * 0.3)) * cos(position.x) * 0.03;
}

void main()
{
    POS4 worldPos;
	#ifdef AS_ENTITY_RENDERER
		POS4 pos = WORLDVIEWPROJ * POSITION;
		worldPos = pos;
	#else
	    worldPos.xyz = (POSITION.xyz * CHUNK_ORIGIN_AND_SCALE.w) + CHUNK_ORIGIN_AND_SCALE.xyz;
	    worldPos.w = 1.0;
	
	    // Transform to view space before projection instead of all at once to avoid floating point errors
	    // Not required for entities because they are already offset by camera translation before rendering
	    // World position here is calculated above and can get huge
	    POS4 pos = WORLDVIEW * worldPos;
	    pos = PROJ * pos;
	#endif
	
	    gl_Position = pos;
	
	#ifndef BYPASS_PIXEL_SHADER
	    uv0 = TEXCOORD_0;
	    uv1 = TEXCOORD_1;
		color = COLOR;
	#endif
	
	position = POSITION.xyz;
	wpos = worldPos.xyz;
	screenPos = pos.st / (pos.p + 1.0);
	
	float rainTime = (1.0 - pow(FOG_CONTROL.y, 5.0));
	float nightTime = pow(max(min(1.0 - FOG_COLOR.r * 1.5, 1.0), 0.0), 1.2);
	float dayTime = pow(max(min(1.0 - FOG_COLOR.b * 1.2, 1.0), 0.0), 0.5);
	float duskTime = clamp(((1.5 - FOG_COLOR.r * 2.0) + (1.0 - FOG_COLOR.b)) * (1.0 - dot(FOG_COLOR.rgb, vec3(0.31))) * FOG_COLOR.g * 2.0, 0.0, 1.0) * (1.0 - rainTime);
	
	float waterWave = cos(length(abs(position.xyz - 8.0) * 10.0) + TIME * 4.0) * 0.03;
	float underWaterWave = cos(length(abs(position.xyz - 8.0) * 10.0) + TIME * 4.0) * 0.03;
		
	if(distanceRender(4.5) != 1.0) //Reduce lag by IZenta
	{
		//Leaf Animation
		if(isPlant(color) || color.w == 0.0) 
		{
			if(isRain(FOG_CONTROL)) gl_Position.x += customLeafWave(5.0);
			else if(isNight(FOG_COLOR)) gl_Position.x += customLeafWave(1.5);
			else gl_Position.x += customLeafWave(3.0);
		}
		
		//Water Animation
		if(isWater(color)) gl_Position.y += waterWave;
		
		//Underwater Animation
		if(!isNether(FOG_COLOR) && !isWater(color) && isUnderwater(FOG_COLOR)) gl_Position.yx += underWaterWave;
	}
	
	// Find distance from the camera
	#if defined(FOG) || defined(BLEND)
		#ifdef FANCY
			vec3 relPos = -worldPos.xyz;
			float cameraDepth = length(relPos);
		#else
			float cameraDepth = pos.z;
		#endif
	#endif
	
	// Apply fog
	#ifdef FOG
		float len = cameraDepth / RENDER_DISTANCE;
		#ifdef ALLOW_FADE
			len += RENDER_CHUNK_FOG_ALPHA;
		#endif
		
		vec3 fcol = mix(dayFogColor, FOG_COLOR.rgb, dayTime);
		fcol = mix(fcol, nightFogColor, nightTime);
		fcol = mix(fcol, duskFogColor, duskTime);
		fcol = mix(fcol, FOG_COLOR.rgb, rainTime);
		fogColor.rgb = fcol.rgb;
		fogColor.a = clamp((len - FOG_CONTROL.x) / (FOG_CONTROL.y - FOG_CONTROL.x), 0.0, 1.0);
	#endif
	
	// Blended layer (mostly water) magic
	#ifdef BLEND
		//Mega hack: only things that become opaque are allowed to have vertex-driven transparency in the Blended layer...
		//to fix this we'd need to find more space for a flag in the vertex format. color.a is the only unused part
		bool shouldBecomeOpaqueInTheDistance = color.a < 0.95;
		if(shouldBecomeOpaqueInTheDistance) {
			#ifdef FANCY  /////enhance water
				float cameraDist = cameraDepth / FAR_CHUNKS_DISTANCE;
				color = COLOR;
			#else
				// Completely insane, but if I don't have these two lines in here, the water doesn't render on a Nexus 6
				vec4 surfColor = vec4(color.rgb, 1.0);
				color = surfColor;
					
				vec3 relPos = -worldPos.xyz;
				float camDist = length(relPos);
				float cameraDist = camDist / FAR_CHUNKS_DISTANCE;
			#endif //FANCY
			
			float alphaFadeOut = clamp(cameraDist, 0.0, 1.0);
			color.a = mix(color.a, 1.0, alphaFadeOut);
		}
	#endif
	
	#ifndef BYPASS_PIXEL_SHADER
		#ifndef FOG
			// If the FOG_COLOR isn't used, the reflection on NVN fails to compute the correct size of the constant buffer as the uniform will also be gone from the reflection data
			color.rgb += FOG_COLOR.rgb * 0.000001;
		#endif
	#endif
}
